'use strict';

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _crypto = require('crypto');

var _crypto2 = _interopRequireDefault(_crypto);

var _jsonwebtoken = require('jsonwebtoken');

var _jsonwebtoken2 = _interopRequireDefault(_jsonwebtoken);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Security helper for botpress
 *
 * Constructor of following functions
 *
 *   - login(user, password, ip)
 *   - authenticate(token)
 *   - getSecret()
 *
 * It will find or create a secret.key in `dataLocation`, then setup the adminPassword for user login.
 *
 * NOTE: current only valid user name is "admin"
 */
module.exports = function (dataLocation, securityConfig) {

  // reading secret from data or creating new secret
  var secret = '';
  var secretPath = _path2.default.join(dataLocation, 'secret.key');

  var createNewSecret = function createNewSecret() {
    secret = _crypto2.default.randomBytes(256).toString();
    _fs2.default.writeFileSync(secretPath, secret);
    return secret;
  };

  if (_fs2.default.existsSync(secretPath)) {
    secret = _fs2.default.readFileSync(secretPath);
  }

  if (!secret || secret.length < 15) {
    secret = createNewSecret();
  }

  var adminPassword = process.env.BOTPRESS_ADMIN_PASSWORD || securityConfig.password;

  // a per-ip cache that logs login attempts
  var attempts = {};
  var lastCleanTimestamp = new Date();
  var maxAttempts = securityConfig.maxAttempts,
      resetAfter = securityConfig.resetAfter,
      tokenExpiry = securityConfig.tokenExpiry;

  // login function that returns a {success, reason, token} object
  // accounts for number of bad attempts

  var login = function login(user, password) {
    var ip = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'all';

    // reset the cache if time elapsed
    if (new Date() - lastCleanTimestamp >= resetAfter) {
      attempts = {};
      lastCleanTimestamp = new Date();
    }

    if (attempts[ip] >= maxAttempts) {
      return {
        success: false,
        reason: 'too many login attempts, try again later'
      };
    }

    if (typeof user === 'string' && user.toLowerCase() === 'admin' && typeof password === 'string' && password === adminPassword) {
      attempts[ip] = 0;
      return {
        success: true,
        token: _jsonwebtoken2.default.sign({ user: 'admin' }, secret, { expiresIn: tokenExpiry })
      };
    } else {
      attempts[ip] = (attempts[ip] || 0) + 1;
      return {
        success: false,
        reason: 'bad user/password combination'
      };
    }
  };

  /**
   * @param {string} token
   * @return {boolean} whether the token is valid
   */
  var authenticate = function authenticate(token) {
    try {
      var decoded = _jsonwebtoken2.default.verify(token, secret);
      return decoded.user === 'admin';
    } catch (err) {
      return false;
    }
  };

  /**
   * Get secret key
   *
   * @return {string}
   */
  var getSecret = function getSecret() {
    return secret;
  };

  return {
    login: login,
    authenticate: authenticate,
    getSecret: getSecret
  };
};
//# sourceMappingURL=security.js.map